#include "global.h"

#include "GameFrame.hpp"
#include "Game.hpp"
#include "OLBitmap.hpp"
#include "Player.hpp"
#include "Math.hpp"
#include "ClockEnemy.hpp"
#include "GrandClockEnemy.hpp"
#include "Particle.hpp"
#include "GravityParticle.hpp"

static const int carspace = 600;
static const int BORDER = 1;
static const int TRACKS = 2;
static const int PLATFORM = 3;
float GameFrame::trainspeed = 0.5;

GameFrame::GameFrame(Game *game) {
	setGame(game);
	
	// Set up the delta time routines with the default fps of 100.0 //
	FpsCounter::Start( 100.0 );
	
	crosshair = game->rm.load<OLBitmap>("data/crosshair.png")->b;
	//evel = game->rm.load<TileMap>("data/level.xml");
	for (int i=1; i<100; i++) {
		OLBitmap *bitmap = game->rm.load<OLBitmap>("data/ascii/ascii" + ToString(i)+".png");
		if (!bitmap) break;
		tiles.push_back(bitmap->b);
	}
	tracks = game->rm.load<OLBitmap>("data/tracks.png")->b;
	train = game->rm.load<OLBitmap>("data/train.png")->b;
	train2 = game->rm.load<OLBitmap>("data/train2.png")->b;
	trackx = 0;
	m1 = game->rm.load<OLBitmap>("data/bar_meter.png")->b;
	m1bg = game->rm.load<OLBitmap>("data/bar_frame.png")->b;
	m2 = game->rm.load<OLBitmap>("data/bar2_meter.png")->b;
	m2bg = game->rm.load<OLBitmap>("data/bar2_frame.png")->b;
	// left
	Poly *p = new Poly();
	p->Add(Vec2D(0,0));
	p->Add(Vec2D(10,0));
	p->Add(Vec2D(10,550));
	p->Add(Vec2D(0,550));
	level.add(p,Rgba(0.0,0.0,0.4),BORDER);
	p = new Poly();
	// right
	p->Add(Vec2D(790,0));
	p->Add(Vec2D(800,0));
	p->Add(Vec2D(800,550));
	p->Add(Vec2D(790,550));
	level.add(p,Rgba(0.0,0.0,0.4),BORDER);
	// bottom
	p = new Poly();
	p->Add(Vec2D(0,550));
	p->Add(Vec2D(800,550));
	p->Add(Vec2D(800,600));
	p->Add(Vec2D(0,600));
	level.add(p,Rgba(0.0,0.0,0.4),TRACKS);
	// top
	p = new Poly();
	p->Add(Vec2D(50,0));
	p->Add(Vec2D(790,0));
	p->Add(Vec2D(790,10));
	p->Add(Vec2D(50,10));
	level.add(p,Rgba(0.0,0.0,0.4),BORDER);
	
	player = new Player(this);
	
	script.read("data/level.txt");
	node.x=0;
	scriptmark = 0;
	scriptpos = 800;
	
	timerate = 1.0;
	timetobump = 0;
	
	particles2.additive = false;
	
	neuropol = new TextRenderer ( "data/neuropol.ttf", 15, 15  );
	midi= load_midi("data/spark.mid");
	play_midi(midi,1);
	
	trainspeed = 0.5;
	ttrainspeed = 0.5;
	
	deathcount = 0;
}
GameFrame::~GameFrame() {
	delete neuropol;
	delete player;
	destroy_midi(midi);
}

void GameFrame::draw() {
	FpsCounter::NewFrameStarted();	
	
	GfxRend::FillScreen(Rgba::BLACK);
	//Rect(0,0,Canvas::Width(),Canvas::Height()).Draw( Rgba(0.0f,0.0f,0.0f, 0.5f ));
	
	neuropol->SetColor(Rgba(0.0,0.1,0.2));
	static float paralax[600/16];
	for (int i=0; i<600/16; i++) {
		int yy = i*16;
		paralax[i] -= (sin(toRadians((yy-300)*180/300))+1)*timerate;
		if (paralax[i]<-16)
			paralax[i]+=16;
		for (float xx=paralax[i]; xx<800; xx+=16) {
			float ratio = (sin(x+xx*0.01)*cos(y) + cos(y+yy*0.01) + 2) / 4;
			tiles[(int)(ratio*tiles.size())]->Blit(xx,yy);
		}
	}
	
	//level->draw(Vec2D(0,0),tiles);
	//for (float x=trackx/4; x<800; x+=tracks->Width()/2)
	//	tracks->BlitStretched(x,550-tracks->Height(),tracks->Width()/2,tracks->Height(),Tinted(Rgba(0.3,0.3,0.3,0.3)));
	
	for (float x=trackx; x<800; x+=tracks->Width())
		tracks->Blit(x,600-tracks->Height());
	//for (float x=trainx; x<800; x+=carspace)
	//	train->Blit(x,600-train->Height());
		
	level.draw();
	
	enemies.draw();
	player->draw();
	particles2.draw();
	particles.draw();
	shots.draw();
	
	neuropol->SetColor(Rgba(1.0,1.0,1.0));
	for (list<Text>::iterator i = texts.begin(); i != texts.end(); i++)
		neuropol->Print(i->message,(int)i->pos.x,(int)i->pos.y);
	
	crosshair->Blit( mouse_x-crosshair->Width()/2, mouse_y-crosshair->Height()/2);
	
	m1bg->Blit(20,130,0.5);
	int h = (int)(player->health * 23 / 100)*8;
	m1->Blit(20,130,Clipped(0,24+23*8-h,32,h));
	m2bg->Blit(52,130,0.5);
	h = (int)(player->power * 23 / 100)*8;
	m2->Blit(52,130,Clipped(0,24+23*8-h,32,h));
	
	string msg = ToString( (int)FpsCounter::GetFps()) + "fps";
	neuropol->SetColor(Rgba::WHITE);
	neuropol->Print( msg, 800-neuropol->FirstLineWidth(msg)-5,15);
	
	GfxRend::RefreshScreen();
}
void GameFrame::update() {
	
	x += 0.01*timerate;
	y += 0.02*timerate;
	trackx-=timerate*10;
	if (trackx<-tracks->Width())
		trackx+=tracks->Width();
	trainx-=timerate*trainspeed;
	if (trainx<-carspace)
		trainx+=carspace;
	//level.placement.SetPosition(Vec2D(trainx,0));
	enemies.update(timerate);
	player->update(pow(timerate,0.5f));
	particles.update(timerate);
	particles2.update(timerate);
	shots.update(timerate);
	level.update(timerate);
	Bitmap *b = game->rm.load<OLBitmap>("data/particle2.png")->b;
	
	trainspeed += (ttrainspeed-trainspeed)/300;
	
	scriptpos += timerate*trainspeed;
	float nextnodex = (node.x+scriptmark)-scriptpos+800;
	if (nextnodex<=900) {
		gLog.log(Log::vDebug, "script event : x=%f, y=%f, type=%s", node.x,node.y,node.type.c_str());
		if (node.type == "-") {
			scriptmark += node.x;
		} else if (node.type == "car1") {
			Poly *t = new Poly();
			t->Add(Vec2D(nextnodex+27,384));
			t->Add(Vec2D(nextnodex+467,384));
			t->Add(Vec2D(nextnodex+467,600));
			t->Add(Vec2D(nextnodex+27,600));
			level.add(t,Rgba(0.3,0.0,0.0),PLATFORM,train,Vec2D(nextnodex,600-train->Height()));
		} else if (node.type == "car2") {
			Poly *t = new Poly();
			t->Add(Vec2D(nextnodex+32,520));
			t->Add(Vec2D(nextnodex+458,520));
			t->Add(Vec2D(nextnodex+458,600));
			t->Add(Vec2D(nextnodex+32,600));
			level.add(t,Rgba(0.3,0.0,0.0),PLATFORM,train2,Vec2D(nextnodex,600-train2->Height()));
			t = new Poly();
			t->Add(Vec2D(nextnodex+23,460));
			t->Add(Vec2D(nextnodex+113,460));
			t->Add(Vec2D(nextnodex+113,520));
			t->Add(Vec2D(nextnodex+23,520));
			level.add(t,Rgba(0.3,0.0,0.0),PLATFORM);
			t = new Poly();
			t->Add(Vec2D(nextnodex+372,460));
			t->Add(Vec2D(nextnodex+458,460));
			t->Add(Vec2D(nextnodex+458,520));
			t->Add(Vec2D(nextnodex+372,520));
			level.add(t,Rgba(0.3,0.0,0.0),PLATFORM);
		} else if (node.type == "bat") {
			Enemy *e = new ClockEnemy(this,player);
			e->pos.x = nextnodex;
			e->pos.y = node.y;
			enemies.add(e);
		} else if (node.type == "clock") {
			Enemy *e = new GrandClockEnemy(this,player);
			e->pos.x = nextnodex;
			e->pos.y = node.y;
			enemies.add(e);
		} else if (node.type == "text") {
			Text text;
			text.pos = Vec2D(nextnodex,node.y);
			text.message = node.message ;
			text.maxx = nextnodex+neuropol->FirstLineWidth(text.message);
			texts.push_back(text);
		} else if (node.type == "speed") {
			ttrainspeed = node.y;
		}
		
		if (script.hasNext()) {
			node = script.next();
		} else
			node.type = "";
	}
	
	for (list<Text>::iterator i = texts.begin(); i != texts.end(); ) {
		i->pos.x-=trainspeed*timerate;
		i->maxx-=trainspeed*timerate;
		if (i->maxx<0)
			i = texts.erase(i);
		else
			i++;
	}
	for (list<Particle*>::iterator i = shots.particles.begin(); 
			i != shots.particles.end(); i++) {
		Particle *p = *i;
		if (p->opacity<0.3) continue;
		bool hit = false;
		Poly poly;
		poly.Add(p->pos);
		poly.Add(p->pos+p->delta);
		int count = 0;
		int type = 0;
		if (level.getCollision(poly,Placement(),&type).IsCollision()) {
			if (type > BORDER) {
				count = 3;
				p->delta.x = 0;
				p->delta.y = 0;
				p->opacity *= 0.25;
			}
		} else 
			for (list<Enemy*>::iterator i2 = enemies.enemies.begin(); 
					i2 != enemies.enemies.end(); i2++) {
				Enemy *e = *i2;
				int i = e->hit(p->pos,32,10);
				if (i>0) {
					p->delta.x = 0;
					p->delta.y = 0;
					p->opacity *= 0.25;
					count = 10;
					if (i==2)
						count = 100;
					break;
				}
			}		
		for (int i=0; i<count; i++) {
			float randangle = toRadians(rand()%360);
			Vec2D dir(cos(randangle),sin(randangle));
			Particle *p2 = new GravityParticle(p->pos, rand()%100*0.025*dir*cos(randangle*6), 30, b);
			p2->scale = rand()%150 * 0.01;
			particles.add(p2);
		}
	}
	
	if (player->hurt<=0)
		for (list<Enemy*>::iterator i2 = enemies.enemies.begin(); 
				i2 != enemies.enemies.end(); i2++) {
			Enemy *e = *i2;
			int i = e->hit(Vec2D(player->pos.x,player->pos.y-50),50,0);
			if (i>0) {
				player->damage(5);
				
				for (int i=0; i<10; i++) {
					float randangle = toRadians(rand()%360);
					Vec2D dir(cos(randangle),sin(randangle));
					Particle *p2 = new GravityParticle(e->pos, rand()%100*0.025*dir*cos(randangle*6), 30, b);
					p2->scale = rand()%150 * 0.01;
					particles.add(p2);
				}
			}
		}
	
	if (player->health<=0) {
		if (deathcount==0) {
			destroy_midi(midi);
			midi= load_midi("data/death.mid");
			play_midi(midi,0);
			player->delta.x=4;
			player->delta.y=-8;
		}
		if (deathcount>500) {
			// OMG SO HAX! (this deletes self)
			game->setFrame(new GameFrame(game));
			return;
		}
		deathcount++;
		for (int i=0; i<10; i++) {
			float randangle = toRadians(rand()%360);
			Vec2D dir(cos(randangle),sin(randangle));
			Particle *p2 = new Particle(player->pos, rand()%400*0.025*dir*cos(randangle*6), 30, b);
			p2->scale = rand()%150 * 0.01;
			particles.add(p2);
		}
	}
	if (key[KEY_SPACE]) {
		player->power -= 0.5;
		if (player->power<0)
			player->power=0;
	} else if (player->power<100) {
		player->power+=0.1;
	}
	ttimerate = (key[KEY_SPACE]&&player->power>0) ? 0.1 : 1.0;
	timerate += (ttimerate-timerate)/40;
	Transforms::SetTintColor( Rgba(0.7f, 0.7f, 0.7f , (1.0f-timerate)*0.6f) ); 
	if (key[KEY_ESC])
		game->stop();
}
