#include "TileMap.hpp"
#include "partypack/dom.h"

#include "global.h"

using namespace xdom;


TileMap::TileMap() {
	tiles = NULL;
}

TileMap::TileMap(int _width, int _height) : width(_width), height(_height) {
	tiles = new int[width*height];
}
TileMap::~TileMap() {
	delete [] tiles;
}
int TileMap::getTile(int x, int y) {
	if (x<0 || y<0 || x>=width || y>=height || !tiles) return -1;
	return tiles[x+y*width];
}
void TileMap::setTile(int x, int y, int tile) {
	if (x<0 || y<0 || x>=width || y>=height || !tiles) return;
	tiles[x+y*width] = tile;
}
void TileMap::setSize(int _width, int _height) {
	width = _width;
	height = _height;
	
	if (tiles)
		delete [] tiles;
	tiles = new int[width*height];
	
}
static int toHex(char c) {
	if (c>='0' && c<='9')
		return c-'0';
	if (c>='a' && c<='f')
		return c-'a'+10;
	if (c>='A' && c<='F')
		return c-'A'+10;
	return -1;
}
bool TileMap::load(int width, int height, std::string map) {
	setSize(width,height);
	int x=0, y=0;
	for (int i=0; i<map.size(); i++) {
		char c = map[i];
		switch (c) {
			case ' ':
				setTile(x,y,0);
				x++;
				i++;
				break;
			case '\n':
				y++;
				if (x>width || y>height)
					return load(x,max(y,height),map);
				x=0;
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
			case 'a':
			case 'b':
			case 'c':
			case 'd':
			case 'e':
			case 'f':
				setTile(x,y,toHex(c)*16 + toHex(map[++i]));
				x++;
				break;
				
				
		}
	}
	return true;
	
}
bool TileMap::load(std::string file) {

	string result("");
	gLog.log(Log::vDebug,"map %s",file.c_str());
	
	char buffer[4096];
	FILE *fp = fopen(file.c_str(), "r");
	if (!fp) 
		return false;
	
	while(fgets(buffer, 4096, fp))
	{
		result += buffer;
	}
	fclose(fp);
	
	DomParser parser = DomParser(result);
	DomNode node = parser.parse();
	
	DomNode map = node.getFirstNamedChild("map");
	
	
	if (map.cdata.size()) {
		string data("");
		
		for (int i=0; i<map.cdata.size(); i++)
			data += map.cdata[i];
		
		return load(1,1,data);
	}
	
	return false;
}

void TileMap::draw(Vec2D pos, vector<Bitmap*>tiles) {
	int max = tiles.size();
	if (max==0) return;
	int tilewidth = tiles[0]->Width();
	int tileheight = tiles[0]->Height();
	for (int y=0; y<height; y++)
		for (int x=0; x<width; x++) {
			int tile = getTile(x,y)-1;
			if (tile>=0 && tile<max)
				tiles[tile]->Blit(x*tilewidth-pos.x,y*tileheight-pos.y);
		}
}

